#include <p18cxxx.h>
#include <portb.h>
#pragma config WDT = OFF, OSC = INTIO2, FSCM = OFF, IESO = OFF, LVP = OFF, MCLRE = OFF, STVR = OFF, DEBUG = OFF, WRT0 = OFF, WRT1 = OFF, CP0 = OFF, CP1 = OFF, CPD = OFF, BOR = OFF

#include <adc.h>//include library ADC functions
#include <delays.h>//delay functions
#include <usart.h>//include serial functions
#include <stdlib.h>//include string conversion functions
#include <math.h>//include math functions





#define LEDPIN LATAbits.LATA0 //A #define command basically makes the compiler do a find-and-replace, 
							//so wherever I write "LEDPIN" it will replace it with LATAbits.LATA0 before
							//compiling. I like to do this to make programs easier to read. Also, if I change
							//which pin is the LED pin, I only have to change it at one location.

#define ADCMAX 1023 //this is the maximum value the ADC result can have, 2^10 - 1.I know this because it's a 10-bit 
					//ADC, so the highest number it can hold is 1111111111 = 1023.
					//The maximum number will correspond to 5V because I configure it below to use the power supply
					//pin (5V) as its reference maximum voltage. (analog to digital converters need a high and low
					//reference voltage to operate. Then they produce a number which represents where the input
					//is relative to the reference voltages. For example, a 2.5V input will produce a value of
					//about 512, because it's halfway between 0V and 5V.

//Now declare the variables we'll use. C18 needs variables to be declared ahead of time. It gives a syntax
//error if you declare new variables in the middle of the main() function.
int ADCresult;
float voltage;


int intvoltage;
int decvoltage;
char outputstring[20];
char decimalstring[6];

void main (void)
{
	//*****begin standard config*****
	OSCCON = 0x70;	//set it to use the 8MHz internal clock.
	EnablePullups(); //PORTB pullup resistors enabled. Doesn't really matter for this tutorial.
	//*****end standard config*****

	//****begin project specific config
	ADCON1 = 0b01111101;	//set all ADC-capable pins to NOT be analog inputs, just digital IO, except for AN1 (the leftmost bit is meaningless)
	DDRB=0b11111101;  //all inputs, except serial TX is RB1, so that has to be an output
	DDRA=0b00000010; //all outputs except RA1, which is AN1
					//the order is RA7 - RA0.
  	//pin direction can also be set individually with commands like: DDRAbits.RA0=0;

	//****end project-specific config***


	//Configure the serial port. See library function for what each term is doing.
	//This is a pretty standard serial setup.
	OpenUSART(USART_TX_INT_OFF &
  			USART_RX_INT_OFF &
  			USART_ASYNCH_MODE &
  			USART_EIGHT_BIT &
  			USART_CONT_RX &
  			USART_BRGH_LOW,
  			12);		//this is the baud rate number. Baudrate with BRGH_LOW is 
  					//Fosc/(64*(baud rate number + 1))
  					//The number 12 should give 9600 baud if clock speed is 8MHz.








	//First, use the library OpenADC function to "open" and configure the ADC peripheral. Note that this PIC only has one
	//ADC peripheral, even though there are 7 pins that it can connect to. In other words, we can only do
	//one analog-to-digital conversion at a time.
	OpenADC( ADC_FOSC_32    &  //This was recommended in Table 17-2 of the 18LF1320 datasheet, for clock speeds up to 10.65MHz
	    	 ADC_RIGHT_JUST &  //the result will be in the lowest 10 bits.
	     	 ADC_4_TAD, //The 18F1320 datasheet recommended this setting, if I read the datasheet right.
	    	 ADC_CH1  &  //Use pin AN1 i.e. RA1
	    	 ADC_VREFPLUS_VDD & //Set the reference maximum voltage to be the + power supply voltage (5V) and the
	    	 ADC_VREFMINUS_VSS &//reference minimum to the (-) power supply voltage (0V).
	    	 ADC_INT_OFF, 0b01111101  ); //Don't set an interrupt, and set only AN1 to be an analog pin (this was also done above, at the beginning of main(), but the library function does it here too.)

	//PortBbits.RB3
  	Delay10KTCYx(200);//will delay 2 million instruction cycles, or 1 second for 8MHz clock
					//(since 4 clock cycles makes 1 instruction cycle).
					//I added the delay to make sure the MAX232 chip has time to start up.

	putrsUSART("Started up");//for some reason this gives a compiler warning, but still works.
  	
  	while(1)
  	{
	  	ConvertADC(); //This makes it "capture" the analog voltage and start the analog to digital conversion process.
	  	while(BusyADC() )//BusyADC() is true until the conversion process completes. Need to wait for it to complete or we'll get bad data.
	  	{//do nothing and wait for conversion to happen.
		}
		ADCresult = ReadADC();//When the while loop exits, we know the conversion is done. So get the result from the ADC peripheral.
		voltage = (double)ADCresult *(5.0)/ ADCMAX;//Scale it into a voltage by multiplying by 5V/1023
													//The "(double)" forces it to do floating-point math, i.e. give us decimals,
													//rather than integer math.
		//now we have the voltage and can do whatever we want. Let's light an LED if it's greater than 2.8V
		if(voltage > 2.8)
			LEDPIN = 1;  //note that I defined LEDPIN above
		else
			LEDPIN = 0;


		//start of new serial code.
		//first, convert things to a character array:
		intvoltage = floor(voltage);//chop off whatever is right of the decimal
		voltage = voltage - intvoltage;//find what's left
		voltage = voltage * 1000;//move the left 3 digits to the left of the decimal
		decvoltage = floor(voltage);//chop off whatever is right of the decimal

		itoa(intvoltage, &outputstring);//converts an integer to an array of characters.
										//the characters will be stored in "outputstring"
										//The "&" means "memory address of" which is what the 
										//function needs.
		itoa(decvoltage, &decimalstring);

		//actual serial transmission:
		putrsUSART("Voltage is   ");
		putsUSART(&outputstring);
		putrsUSART(".");
		putsUSART(&decimalstring);
		putrsUSART(" V\r\n");	//the "\r\n" means "go to the next line" i.e. Enter.
								//For PC programming, \n is enough, but \r\n is the "formal" 
								//way and is apparently required for serial comms.  Wikipedia
								//"newline" for details.
		
		while(BusyUSART())
		{}//this will wait for the serial transmission to finish before starting over.
			//This is not necessary if you're sure your program has enough delays elsewhere.
		Delay10KTCYx(100);//delay another 1/2 second to let a human read the text.

	}
	
}
